/*
 * Copyright (c) 2022 Reza Mahdi. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are hereby granted under terms of BSD 3-Clause License.
 * See LICENSE for more information.
 */

#ifndef DNSPIDER_BENCHMARK_H
#define DNSPIDER_BENCHMARK_H

#include <time.h>

/**
 * @brief timing statisitc of an operation
 */
typedef struct
{
  struct timespec min; /**< minimum time duration */
  struct timespec avg; /**< average time duration */
  struct timespec max; /**< maximum time duration */
} benchmark_timing_t;

/**
 * @brief rating of operation as count per second
 */
typedef struct
{
  long int min; /**< minimum times per second */
  long int avg; /**< average times per second */
  long int max; /**< maximum times per second */
} benchmark_rating_t;

/**
 * @brief a benchmark
 */
typedef struct
{
  char *name;                /**< name of benchmark */
  benchmark_timing_t timing; /**< timing of benchmark */
  benchmark_rating_t rating; /**< rating of benchmark */
} benchmark_t;

/**
 * @brief calculate the difference between to time points
 *
 * @param start staring point of time
 * @param end end of time duration
 * @param diff timespec to store difference in
 */
void
timespec_diff (struct timespec *start, struct timespec *end,
               struct timespec *diff)
{
  if ((end->tv_nsec - start->tv_nsec) < 0)
    {
      diff->tv_sec = end->tv_sec - start->tv_sec - 1;
      diff->tv_nsec = 1000000000 + end->tv_nsec - start->tv_nsec;
    }
  else
    {
      diff->tv_sec = end->tv_sec - start->tv_sec;
      diff->tv_nsec = end->tv_nsec - start->tv_nsec;
    }
}
#endif
