/*
 * Copyright (c) 2022 Reza Mahdi. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are hereby granted under terms of BSD 3-Clause License.
 * See LICENSE for more information.
 */

#include "list.h"

void
list_init (list_t *list)
{
  list->head = NULL;
  list->tail = NULL;
  list->elements = 0;
}

void
list_append (list_t *list, list_head_t *element)
{
  element->next = NULL;
  if (list->head == NULL)
    {
      list->head = element;
      list->tail = element;
      element->prev = NULL;
    }
  else
    {
      list->head->next = element;
      element->prev = list->head;
      list->head = element;
    }
  list->elements++;
}

void
list_prepend (list_t *list, list_head_t *element)
{
  element->prev = NULL;
  if (list->tail == NULL)
    {
      list->tail = element;
      list->head = element;
      element->next = NULL;
    }
  else
    {
      list->tail->prev = element;
      element->next = list->tail;
      list->tail = element;
    }
  list->elements++;
}

list_head_t *
list_pop_back (list_t *list)
{
  list_head_t *item;

  item = list->head;

  if (item == NULL)
    {
      return NULL;
    }

  list->head = item->prev;
  if (list->head != NULL)
    {
      list->head->next = NULL;
    }

  item->next = NULL;
  item->prev = NULL;

  list->elements--;

  return item;
}

list_head_t *
list_pop_front (list_t *list)
{
  list_head_t *item;

  if (item == NULL)
    {
      return NULL;
    }

  item = list->tail;
  list->tail = item->next;
  if (list->tail != NULL)
    {
      list->tail->prev = NULL;
    }

  item->next = NULL;
  item->prev = NULL;

  list->elements--;

  return item;
}

int
list_is_empty (list_t *list)
{
  return list->elements == 0;
}

size_t
list_count (list_t *list)
{
  return list->elements;
}
