/*
 * Copyright (c) 2022 Reza Mahdi. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are hereby granted under terms of BSD 3-Clause License.
 * See LICENSE for more information.
 */

#ifndef DNSPIDER_LOOP_H_
#define DNSPIDER_LOOP_H_

#include <stddef.h>
#include <sys/socket.h>
#include <sys/types.h>

/**
 * @brief A loop to do async IO operations.
 *
 * Some efficient loops is implemented by DNSpider.
 */
typedef struct
{
  char *name;                             /**< name of loop backend */
  size_t size;                            /**< size of loop struct */
  int (*init_options) (const char *opts); /**< initialize loop with opts */
  int (*create_socket) (int, int, int);   /**< create and register a socket */
} loop_t;

loop_t *get_loop_by_name (const char *name);
loop_t *get_best_loop ();
static inline int
init_loop_options (loop_t *loop, const char *args)
{
  return loop->init_options (args);
}
#endif
