/*
 * Copyright (c) 2022 Reza Mahdi. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are hereby granted under terms of BSD 3-Clause License.
 * See LICENSE for more information.
 */

#ifndef DNSPIDER_WORKR_H_
#define DNSPIDER_WORKR_H_

#include "list.h"
#include "loop.h"
#include "queue.h"
#include <pthread.h>

typedef struct
{
  list_head_t head;
  pthread_t thread;    /**< thread of worker */
  loop_t loop_context; /**< the loop that worker will use */
  queue_t out_queue;   /**< output queue (result queue) */
} worker_t;

typedef struct
{
  list_t workers;
  queue_t job_queue;
} worker_pool_t;

static inline void
wait_for_workers (worker_pool_t *pool)
{
  list_t *workers = &pool->workers;
  list_foreach (workers, item, worker_t, head)
  {
    pthread_join (item->thread, NULL);
  }
}
#endif
