/*
 * Copyright (c) 2022 Reza Mahdi. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are hereby granted under terms of BSD 3-Clause License.
 * See LICENSE for more information.
 */

#define TEST_MAIN
#define TEST_SEGFAULT
#include "test.h"

#include <list.h>

TEST (list, empty)
{
  list_t list;
  list_init (&list);

  ASSERT_NULL (list.head);
  ASSERT_NULL (list.tail);
  ASSERT_EQUAL (list.elements, 0);
}

/**** APPEND
 * ******************************************************************/
TEST (list, append_empty)
{
  list_t list;
  list_init (&list);

  list_head_t item;

  list_append (&list, &item);

  ASSERT_NOT_NULL (list.tail);
  ASSERT_NOT_NULL (list.head);
  ASSERT_EQUAL_PTR (list.head, list.tail);
  ASSERT_EQUAL (list.elements, 1);
}

TEST (list, append_nonempty)
{
  list_t list;
  list_head_t item1, item2;

  list_init (&list);
  list_append (&list, &item1);
  list_append (&list, &item2);

  ASSERT_EQUAL_PTR (list.tail, &item1);
  ASSERT_EQUAL_PTR (list.head, &item2);
  ASSERT_EQUAL_PTR (list.elements, 2);

  ASSERT_NULL (item1.prev);
  ASSERT_EQUAL_PTR (item1.next, &item2);
  ASSERT_EQUAL_PTR (item2.prev, &item1);
  ASSERT_NULL (item2.next);
}

/**** PREPEND
 * *****************************************************************/
TEST (list, prepend_empty)
{
  list_t list;
  list_head_t item;

  list_init (&list);
  list_prepend (&list, &item);

  ASSERT_NOT_NULL (list.head);
  ASSERT_NOT_NULL (list.tail);
  ASSERT_EQUAL_PTR (list.head, list.tail);
  ASSERT_EQUAL (list.elements, 1);
}

TEST (list, prepend_nonempty)
{
  list_t list;
  list_head_t item1, item2;

  list_init (&list);
  list_prepend (&list, &item1);
  list_prepend (&list, &item2);

  ASSERT_EQUAL_PTR (list.tail, &item2);
  ASSERT_EQUAL_PTR (list.head, &item1);
  ASSERT_EQUAL (list.elements, 2);

  ASSERT_NULL (item2.prev);
  ASSERT_EQUAL_PTR (item2.next, &item1);
  ASSERT_EQUAL_PTR (item1.prev, &item2);
  ASSERT_NULL (item1.next);
}

/**** POP
 * *********************************************************************/
TEST (list, pop_back_empty)
{
  list_t list;
  list_head_t *item;

  list_init (&list);
  item = list_pop_back (&list);

  ASSERT_NULL (item);
  ASSERT_EQUAL (list.elements, 0);
  ASSERT_NULL (list.head);
  ASSERT_NULL (list.tail);
}

TEST (list, pop_back_nonempty)
{
  list_t list;
  list_head_t item1, item2, *t;

  list_init (&list);
  list_append (&list, &item1);
  list_append (&list, &item2);

  t = list_pop_back (&list);

  ASSERT_EQUAL_PTR (t, &item2);
  ASSERT_EQUAL (list.elements, 1);
  ASSERT_NULL (item2.next);
  ASSERT_NULL (item2.prev);
  ASSERT_NULL (item1.next);
}

TEST (list, pop_front_empty)
{
  list_t list;
  list_head_t *item;

  list_init (&list);
  item = list_pop_front (&list);

  ASSERT_NULL (item);
  ASSERT_EQUAL (list.elements, 0);
  ASSERT_NULL (list.head);
  ASSERT_NULL (list.tail);
}

TEST_SKIP (list, pop_front_nonempty) {}

int
main (int argc, const char *argv[])
{
  int result = test_main (argc, argv);

  return result;
}
